/*------------------------------------------------------------------------ *
 * \file	evmomapl137_mcasp.c                                            *
 * \brief	configure and initialize McASP1 for interrupt-based i/o        *
 * ----------------------------------------------------------------------- */


//  ======== Includes =======================================================
#include "evmomapl137_mcasp.h"

//  ========  Private Defines and Macros ====================================
extern void intcVectorTable( void );

extern	cregister volatile unsigned int	CSR;	// control status register
extern	cregister volatile unsigned int	ICR;	// interrupt clear register
extern	cregister volatile unsigned int	IER;	// interrupt enable register
extern	cregister volatile unsigned int	ISTP;	// interrupt

#define INTC_INTMUX1            *(unsigned int*)(0x01800104)

static MCASP_Handle mcasp;

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *  McASP1 is in MASTER mode.                                               *
 *      BCLK & WCLK come from McASP1                                        *
 *      DIN is used by write16/write32                                      *
 *      DOUT is usec by read16/read32                                       *
 *                                                                          *
 * ------------------------------------------------------------------------ */

//  ========  EVMOMAPL137_PCASP_init =========================================
//  setup McASP registers for Interrupt mode and I2S mode
void EVMOMAPL137_MCASP_init( Int32 fsMcASP )
{
		/* enable write access to PINMUX and CFG registers in KICK0R and KICK1R */
		KICK0R = 0x83E70B13;                      // Kick0 register and data (unlock)
		KICK1R = 0x95A4F1E0;                      // Kick1 register and data (unlock)

		/* write to the PINMUX registers to enable the McASP1 and I2C0 */
		PINMUX8  = 0x01122000u;                   // McASP1 and I2C0
		PINMUX11 = 0x11100000u;                   // McASP1
		PINMUX12 = 0x11111111u;                   // McASP1
		PINMUX13 = 0x00111111u;                   // McASP1
		KICK0R = 0;
		KICK1R = 1;

		/* reset MCASP1 */
	    mcasp = &MCASP_MODULE_1;
	    mcasp->regs->GBLCTL  = 0;       // reset
	    mcasp->regs->RGBLCTL = 0;       // reset RX
	    mcasp->regs->XGBLCTL = 0;       // reset TX
	    mcasp->regs->PWRDEMU = 1;       // free-running

/* ------------------------------------------------------------------------------------ *
 * NOTE: ROR 16-bits enabled for both XMT/RCV. SLOT SIZE = 16 bits, 1-bit delay for Rx  *
 *       clock and frame sync generated by extern oscillator of 24576 KHz               *
 *       Tx and Rx synchronized                                                         *
 * ------------------------------------------------------------------------------------ */

		/* conFiguration RX mode I2S */
	    mcasp->regs->RMASK      = 0xFFFFFFFF; // no padding used
	    mcasp->regs->RFMT       = 0x0001807C; // MSB First, align left, slot=16bits, 1-bit delay, ROR 16-bits
	    mcasp->regs->AFSRCTL    = 0x00000112; // frame sync generated externally, FS/word, 2 SLOT TDM = I2S
	    mcasp->regs->RTDM       = 0x00000003; // slots 0,1
	    mcasp->regs->RINTCTL    = 0x00000000; // not used
	    mcasp->regs->RCLKCHK    = 0x00FF0008; // 255-MAX 0-MIN, div-by-256

		/* configuration TX mode I2S */
	    mcasp->regs->XMASK      = 0xFFFFFFFF; // no padding used
	    mcasp->regs->XFMT       = 0x0001807C; // MSB First, align left, slot=16bits, 1-bit delay, ROR 16-bits
	    mcasp->regs->AFSXCTL    = 0x00000112; // frame sync generated externally, FS/word, 2 SLOT TDM = I2S
	    mcasp->regs->XTDM       = 0x00000003; // slots 0,1
	    mcasp->regs->XINTCTL    = 0x00000020; // enable interrupt on transmit
	    mcasp->regs->XCLKCHK    = 0x00FF0008; // 255-MAX 0-MIN, div-by-256

		/* configuration clock operation */
	    switch( fsMcASP )					  // set up sampling rate - default 8kHz
	    {
	    	case FS_8000_HZ:
	    	    mcasp->regs->ACLKRCTL   = 0x000000A7; // rising INTERNAL CLK,(From tx side)
	    	    mcasp->regs->ACLKXCTL   = 0x000000A7; // ASYNC, Rising INTERNAL CLK, div-by-96
	    	    break;
	    	case FS_12000_HZ:
	    		mcasp->regs->ACLKRCTL   = 0x000000A9; // rising INTERNAL CLK,(From tx side)
	    		mcasp->regs->ACLKXCTL   = 0x000000A9; // ASYNC, Rising INTERNAL CLK, div-by-64
	    		break;
	    	case FS_16000_HZ:
	    		mcasp->regs->ACLKRCTL   = 0x000000AF; // rising INTERNAL CLK,(From tx side)
	    		mcasp->regs->ACLKXCTL   = 0x000000AF; // ASYNC, Rising INTERNAL CLK, div-by-48
	    		break;
	    	case FS_24000_HZ:
	    		mcasp->regs->ACLKRCTL   = 0x000000AD; // rising INTERNAL CLK,(From tx side)
	    		mcasp->regs->ACLKXCTL   = 0x000000AD; // ASYNC, Rising INTERNAL CLK, div-by-32
	    		break;
	    	case FS_48000_HZ:
	    		mcasp->regs->ACLKRCTL   = 0x000000AF; // rising INTERNAL CLK,(From tx side)
	    		mcasp->regs->ACLKXCTL   = 0x000000AF; // ASYNC, Rising INTERNAL CLK, div-by-16
	    		break;
	    	default:
	    		mcasp->regs->ACLKRCTL   = 0x000000A7; // rising INTERNAL CLK,(From tx side)
	    		mcasp->regs->ACLKXCTL   = 0x000000A7; // ASYNC, Rising INTERNAL CLK, div-by-8
	    		break;
	    }

	    mcasp->regs->AHCLKRCTL  = 0x00000000; // INT CLK (From tx side)
	    mcasp->regs->AHCLKXCTL  = 0x00000000; // INT CLK, div-by-4

		/* conFiguration serializers (5 = xmit , 0 = rcv) */
	    mcasp->regs->SRCTL5     = 0x000D;     // MCASP1.AXR1[5] --> DIN
	    mcasp->regs->SRCTL0     = 0x000E;     // MCASP1.AXR1[0] <-- DOUT

		/* conFiguration pin Function and direction */
	    mcasp->regs->PFUNC      = 0;          // All MCASPs
	    mcasp->regs->PDIR       = 0x14000020; // ACLK and AFSX out_datas pins
	    mcasp->regs->DITCTL     = 0x00000000; // not used
	    mcasp->regs->DLBCTL     = 0x00000000; // not used
	    mcasp->regs->AMUTE      = 0x00000000; // not used

	    return;
}

//  ========  EVMOMAPL137_MCASP_start ==========================================
//  enable the audio clocks and verification that each bit is properly set
void EVMOMAPL137_MCASP_start( void )
{
    /* starting sections oF the McASP */
    mcasp->regs->XGBLCTL |= GBLCTL_XHCLKRST_ON;                                    // HS Clk
    while ( ( mcasp->regs->XGBLCTL & GBLCTL_XHCLKRST_ON ) != GBLCTL_XHCLKRST_ON );
    mcasp->regs->RGBLCTL |= GBLCTL_RHCLKRST_ON;                                    // HS Clk
    while ( ( mcasp->regs->RGBLCTL & GBLCTL_RHCLKRST_ON ) != GBLCTL_RHCLKRST_ON );

    mcasp->regs->XGBLCTL |= GBLCTL_XCLKRST_ON;                                     // Clk
    while ( ( mcasp->regs->XGBLCTL & GBLCTL_XCLKRST_ON ) != GBLCTL_XCLKRST_ON );
    mcasp->regs->RGBLCTL |= GBLCTL_RCLKRST_ON;                                     // Clk
    while ( ( mcasp->regs->RGBLCTL & GBLCTL_RCLKRST_ON ) != GBLCTL_RCLKRST_ON );

    mcasp->regs->XSTAT = 0x0000FFFF;        // clear all
    mcasp->regs->RSTAT = 0x0000FFFF;        // clear all

    mcasp->regs->XGBLCTL |= GBLCTL_XSRCLR_ON;                                      // serialize
    while ( ( mcasp->regs->XGBLCTL & GBLCTL_XSRCLR_ON ) != GBLCTL_XSRCLR_ON );
    mcasp->regs->RGBLCTL |= GBLCTL_RSRCLR_ON;                                      // serialize
    while ( ( mcasp->regs->RGBLCTL & GBLCTL_RSRCLR_ON ) != GBLCTL_RSRCLR_ON );

    /* write a 0, so that no underrun occurs aFter releasing the state machine */
    mcasp->regs->XBUF5 = 0;
    mcasp->regs->RBUF0 = 0;

    mcasp->regs->XGBLCTL |= GBLCTL_XSMRST_ON;                                       // state Machine
    while ( ( mcasp->regs->XGBLCTL & GBLCTL_XSMRST_ON ) != GBLCTL_XSMRST_ON );
    mcasp->regs->RGBLCTL |= GBLCTL_RSMRST_ON;                                       // state Machine
    while ( ( mcasp->regs->RGBLCTL & GBLCTL_RSMRST_ON ) != GBLCTL_RSMRST_ON );

    mcasp->regs->XGBLCTL |= GBLCTL_XFRST_ON;                                        // frame Sync
    while ( ( mcasp->regs->XGBLCTL & GBLCTL_XFRST_ON ) != GBLCTL_XFRST_ON );
    mcasp->regs->RGBLCTL |= GBLCTL_RFRST_ON;                                        // frame Sync
    while ( ( mcasp->regs->RGBLCTL & GBLCTL_RFRST_ON ) != GBLCTL_RFRST_ON );

    /* start by sending a dummy write */
    while( ! ( mcasp->regs->SRCTL5 & 0x10 ) );  // check For Tx ready
    mcasp->regs->XBUF5 = 0;

    return;

}

//  ======== EVOMAPL137_intr_init ===========================================
//  enable the interrupt mode and configuration PINMUX for McASP1
void EVMOMAPL137_MCASP_isr( void )
{
	     /* configuration interrupt register of McASP */
	     CSR = 0x0000;                             // disable interrupts globally while initializing
	     INTC_INTMUX1 = 0x3D;					   // associate event 61 (McASP event) with interrupt 4 by writing 61 into LSBs of INTMUX1
	     ISTP = (unsigned int)intcVectorTable;
	     ICR = 0xFFF0;                             // clear bits in interrupt flag  register  IFR
	     IER |= 0x0012;							   // enable RTDX interrupts NMI (bit 1) and INT4 (bit 4)
	     CSR |= 0x0001;							   // enable interrupts globally
	     return;
}
